@@ -18,7 +18,6 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- * 
  */
 
 #include <string.h>
@@ -28,25 +27,21 @@
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
+#include <dlfcn.h>
 #include <glib.h>
-#include <glib-object.h>
-#include <dbus/dbus-glib.h>
 #include <stdlib.h>
 
+#include "nm-fortisslvpn-pppd-service-dbus.h"
 #include "nm-fortisslvpn-service.h"
 #include "nm-ppp-status.h"
 
 #include <nm-utils.h>
 
-/* These are here because nm-dbus-glib-types.h isn't exported */
-#define DBUS_TYPE_G_ARRAY_OF_UINT          (dbus_g_type_get_collection ("GArray", G_TYPE_UINT))
-#define DBUS_TYPE_G_ARRAY_OF_ARRAY_OF_UINT (dbus_g_type_get_collection ("GPtrArray", DBUS_TYPE_G_ARRAY_OF_UINT))
-
 int plugin_init (void);
 
 char pppd_version[] = VERSION;
 
-static DBusGProxy *proxy = NULL;
+static NMDBusFortisslvpnPpp *proxy = NULL;
 
 static void
 nm_phasechange (void *data, int arg)
@@ -54,7 +49,7 @@ nm_phasechange (void *data, int arg)
 	NMPPPStatus ppp_status = NM_PPP_STATUS_UNKNOWN;
 	char *ppp_phase;
 
-	g_return_if_fail (DBUS_IS_G_PROXY (proxy));
+	g_return_if_fail (NMDBUS_IS_FORTISSLVPN_PPP_PROXY (proxy));
 
 	switch (arg) {
 	case PHASE_DEAD:
@@ -121,57 +116,24 @@ nm_phasechange (void *data, int arg)
 	           ppp_phase);
 
 	if (ppp_status != NM_PPP_STATUS_UNKNOWN) {
-		dbus_g_proxy_call_no_reply (proxy, "SetState",
-		                            G_TYPE_UINT, ppp_status,
-		                            G_TYPE_INVALID,
-		                            G_TYPE_INVALID);
+		nmdbus_fortisslvpn_ppp_call_set_state (proxy,
+		                                       ppp_status,
+		                                       NULL,
+		                                       NULL, NULL);
 	}
 }
 
-static GValue *
-str_to_gvalue (const char *str)
-{
-	GValue *val;
-
-	val = g_slice_new0 (GValue);
-	g_value_init (val, G_TYPE_STRING);
-	g_value_set_string (val, str);
-
-	return val;
-}
-
-static GValue *
-uint_to_gvalue (guint32 i)
-{
-	GValue *val;
-
-	val = g_slice_new0 (GValue);
-	g_value_init (val, G_TYPE_UINT);
-	g_value_set_uint (val, i);
-
-	return val;
-}
-
-static void
-value_destroy (gpointer data)
-{
-	GValue *val = (GValue *) data;
-
-	g_value_unset (val);
-	g_slice_free (GValue, val);
-}
-
-static GValue *
+static GVariant *
 get_ip4_routes (in_addr_t ouraddr)
 {
-	GPtrArray *routes;
-	GValue *value = NULL;
+	GVariantBuilder builder;
+	GVariant *value;
 	int i;
 
-	routes = g_ptr_array_new ();
+	g_variant_builder_init (&builder, G_VARIANT_TYPE ("aau"));
 
 	for (i = 0; i < 100; i++) {
-		GArray *array;
+		GVariantBuilder array;
 		gchar *var;
 		const gchar *str;
 		in_addr_t dest, gateway;
@@ -208,23 +170,20 @@ get_ip4_routes (in_addr_t ouraddr)
 		else
 			metric = atol (str);
 
-		array = g_array_sized_new (FALSE, TRUE, sizeof (guint32), 4);
-		g_array_append_val (array, dest);
-		g_array_append_val (array, prefix);
-		g_array_append_val (array, gateway);
-		g_array_append_val (array, metric);
-		g_ptr_array_add (routes, array);
+		g_variant_builder_init (&array, G_VARIANT_TYPE ("au"));
+		g_variant_builder_add_value (&array, g_variant_new_uint32 (dest));
+		g_variant_builder_add_value (&array, g_variant_new_uint32 (prefix));
+		g_variant_builder_add_value (&array, g_variant_new_uint32 (gateway));
+		g_variant_builder_add_value (&array, g_variant_new_uint32 (metric));
+		g_variant_builder_add_value (&builder, g_variant_builder_end (&array));
 	}
 
-	if (routes->len > 0) {
-		value = g_new0 (GValue, 1);
-		g_value_init (value, DBUS_TYPE_G_ARRAY_OF_ARRAY_OF_UINT);
-		g_value_take_boxed (value, routes);
-	} else {
-		g_ptr_array_free (routes, TRUE);
-	}
+	value = g_variant_builder_end (&builder);
+	if (i > 0)
+		return value;
 
-	return value;
+	g_variant_unref (value);
+	return NULL;
 }
 
 static void
@@ -233,90 +192,98 @@ nm_ip_up (void *data, int arg)
 	guint32 pppd_made_up_address = htonl (0x0a404040 + ifunit);
 	ipcp_options opts = ipcp_gotoptions[0];
 	ipcp_options peer_opts = ipcp_hisoptions[0];
-	GHashTable *hash;
-	GArray *array;
-	GValue *val;
+	GVariantBuilder builder;
 	const gchar *str;
+	GVariant *val;
 
-	g_return_if_fail (DBUS_IS_G_PROXY (proxy));
+	g_return_if_fail (NMDBUS_IS_FORTISSLVPN_PPP_PROXY (proxy));
 
 	g_message ("nm-fortisslvpn-ppp-plugin: (%s): ip-up event", __func__);
 
 	if (!opts.ouraddr) {
 		g_warning ("nm-fortisslvpn-ppp-plugin: (%s): didn't receive an internal IP from pppd!", __func__);
+		nm_phasechange (NULL, PHASE_DEAD);
 		return;
 	}
 
-	hash = g_hash_table_new_full (g_str_hash, g_str_equal,
-							NULL, value_destroy);
+	g_variant_builder_init (&builder, G_VARIANT_TYPE_VARDICT);
 
-	g_hash_table_insert (hash, NM_VPN_PLUGIN_IP4_CONFIG_TUNDEV, 
-					 str_to_gvalue (ifname));
+	g_variant_builder_add (&builder, "{sv}",
+	                       NM_VPN_PLUGIN_IP4_CONFIG_TUNDEV,
+	                       g_variant_new_string (ifname));
 
 	str = g_getenv ("VPN_GATEWAY");
 	if (str) {
-		g_hash_table_insert (hash, NM_VPN_PLUGIN_IP4_CONFIG_EXT_GATEWAY,
-		                           uint_to_gvalue (inet_addr (str)));
+		g_variant_builder_add (&builder, "{sv}",
+		                       NM_VPN_PLUGIN_IP4_CONFIG_EXT_GATEWAY,
+		                       g_variant_new_uint32 (inet_addr (str)));
 	} else {
 		g_warning ("nm-fortisslvpn-ppp-plugin: (%s): no external gateway!", __func__);
 	}
 
 	val = get_ip4_routes (opts.ouraddr);
 	if (val)
-		g_hash_table_insert (hash, NM_VPN_PLUGIN_IP4_CONFIG_ROUTES, val);
+		g_variant_builder_add (&builder, "{sv}", NM_VPN_PLUGIN_IP4_CONFIG_ROUTES, val);
 
 	/* Prefer the peer options remote address first, _unless_ pppd made the
 	 * address up, at which point prefer the local options remote address,
 	 * and if that's not right, use the made-up address as a last resort.
 	 */
 	if (peer_opts.hisaddr && (peer_opts.hisaddr != pppd_made_up_address)) {
-		g_hash_table_insert (hash, NM_VPN_PLUGIN_IP4_CONFIG_PTP,
-		                     uint_to_gvalue (peer_opts.hisaddr));
+		g_variant_builder_add (&builder, "{sv}",
+		                       NM_VPN_PLUGIN_IP4_CONFIG_PTP,
+		                       g_variant_new_uint32 (peer_opts.hisaddr));
 	} else if (opts.hisaddr) {
-		g_hash_table_insert (hash, NM_VPN_PLUGIN_IP4_CONFIG_PTP,
-		                     uint_to_gvalue (opts.hisaddr));
+		g_variant_builder_add (&builder, "{sv}",
+		                       NM_VPN_PLUGIN_IP4_CONFIG_PTP,
+		                       g_variant_new_uint32 (opts.hisaddr));
 	} else if (peer_opts.hisaddr == pppd_made_up_address) {
 		/* As a last resort, use the made-up address */
-		g_hash_table_insert (hash, NM_VPN_PLUGIN_IP4_CONFIG_PTP,
-		                     uint_to_gvalue (peer_opts.hisaddr));
+		g_variant_builder_add (&builder, "{sv}",
+		                       NM_VPN_PLUGIN_IP4_CONFIG_PTP,
+		                       g_variant_new_uint32 (peer_opts.hisaddr));
 	}
 
-	g_hash_table_insert (hash, NM_VPN_PLUGIN_IP4_CONFIG_ADDRESS, 
-					 uint_to_gvalue (opts.ouraddr));
+	g_variant_builder_add (&builder, "{sv}",
+	                       NM_VPN_PLUGIN_IP4_CONFIG_ADDRESS,
+	                       g_variant_new_uint32 (opts.ouraddr));
 
-	g_hash_table_insert (hash, NM_VPN_PLUGIN_IP4_CONFIG_PREFIX, uint_to_gvalue (32));
+	g_variant_builder_add (&builder, "{sv}",
+	                       NM_VPN_PLUGIN_IP4_CONFIG_PREFIX,
+	                       g_variant_new_uint32 (32));
 
 	if (opts.dnsaddr[0] || opts.dnsaddr[1]) {
-		array = g_array_new (FALSE, FALSE, sizeof (guint32));
+		guint32 dns[2];
+		int len = 0;
 
 		if (opts.dnsaddr[0])
-			g_array_append_val (array, opts.dnsaddr[0]);
+			dns[len++] = opts.dnsaddr[0];
 		if (opts.dnsaddr[1])
-			g_array_append_val (array, opts.dnsaddr[1]);
+			dns[len++] = opts.dnsaddr[1];
 
-		val = g_slice_new0 (GValue);
-		g_value_init (val, DBUS_TYPE_G_UINT_ARRAY);
-		g_value_set_boxed (val, array);
-
-		g_hash_table_insert (hash, NM_VPN_PLUGIN_IP4_CONFIG_DNS, val);
+		g_variant_builder_add (&builder, "{sv}",
+		                       NM_VPN_PLUGIN_IP4_CONFIG_DNS,
+		                       g_variant_new_fixed_array (G_VARIANT_TYPE_UINT32,
+		                                                  dns, len, sizeof (guint32)));
 	}
 
 	/* Default MTU to 1400, which is also what Windows XP/Vista use */
-	g_hash_table_insert (hash, NM_VPN_PLUGIN_IP4_CONFIG_MTU, uint_to_gvalue (1400));
+	g_variant_builder_add (&builder, "{sv}",
+	                       NM_VPN_PLUGIN_IP4_CONFIG_MTU,
+	                       g_variant_new_uint32 (1400));
 
 	g_message ("nm-fortisslvpn-ppp-plugin: (%s): sending Ip4Config to NetworkManager-fortisslvpn...", __func__);
 
-	dbus_g_proxy_call_no_reply (proxy, "SetIp4Config",
-	                            DBUS_TYPE_G_MAP_OF_VARIANT, hash, G_TYPE_INVALID,
-	                            G_TYPE_INVALID);
-
-	g_hash_table_destroy (hash);
+	nmdbus_fortisslvpn_ppp_call_set_ip4_config (proxy,
+	                                            g_variant_new ("(a{sv})", &builder),
+	                                            NULL,
+	                                            NULL, NULL);
 }
 
 static void
 nm_exit_notify (void *data, int arg)
 {
-	g_return_if_fail (DBUS_IS_G_PROXY (proxy));
+	g_return_if_fail (G_IS_DBUS_PROXY (proxy));
 
 	g_message ("nm-fortisslvpn-ppp-plugin: (%s): cleaning up", __func__);
 
@@ -327,18 +294,26 @@ nm_exit_notify (void *data, int arg)
 int
 plugin_init (void)
 {
-	DBusGConnection *bus;
 	GError *err = NULL;
+	const char *bus_name;
 
 #if !GLIB_CHECK_VERSION (2, 35, 0)
 	g_type_init ();
 #endif
 
+	bus_name = getenv ("NM_DBUS_SERVICE_FORTISSLVPN");
+	if (!bus_name)
+		bus_name = NM_DBUS_SERVICE_FORTISSLVPN;
+
 	g_message ("nm-fortisslvpn-ppp-plugin: (%s): initializing", __func__);
 
-	bus = dbus_g_bus_get (DBUS_BUS_SYSTEM, &err);
-	if (!bus) {
-		g_warning ("nm-fortisslvpn-pppd-plugin: (%s): couldn't connect to system bus: (%d) %s",
+        proxy = nmdbus_fortisslvpn_ppp_proxy_new_for_bus_sync (G_BUS_TYPE_SYSTEM,
+                                                               G_DBUS_PROXY_FLAGS_NONE,
+                                                               bus_name,
+                                                               NM_DBUS_PATH_FORTISSLVPN_PPP,
+                                                               NULL, &err);
+	if (!proxy) {
+		g_warning ("nm-fortisslvpn-pppd-plugin: (%s): couldn't create D-Bus proxy: (%d) %s",
 		           __func__,
 		           err ? err->code : -1,
 		           err && err->message ? err->message : "(unknown)");
@@ -346,13 +321,6 @@ plugin_init (void)
 		return -1;
 	}
 
-	proxy = dbus_g_proxy_new_for_name (bus,
-								NM_DBUS_SERVICE_FORTISSLVPN_PPP,
-								NM_DBUS_PATH_FORTISSLVPN_PPP,
-								NM_DBUS_INTERFACE_FORTISSLVPN_PPP);
-
-	dbus_g_connection_unref (bus);
-
 	add_notifier (&phasechange, nm_phasechange, NULL);
 	add_notifier (&ip_up_notifier, nm_ip_up, NULL);
 	add_notifier (&exitnotify, nm_exit_notify, proxy);
